// Bapple2 Global Variables (externs for globals.h)
///////////////////////////////////////////////////
#include "allegro.h"
#include "pathext.h"
#include "bapple2.h"
#include "map.h"
#include "units.h"
#include "radar.h"

// Some macros to help track memory usage
#define calloc(a,b) calloc(a,b), heapused += b
#define malloc(a) malloc(a), heapused += a
//#define cmalloc(a) ({ void *b = cmalloc (a); heapused += *(int *)(b-4); b; })
//#define malloc(a) ({ void *b = malloc (a); heapused += *(int *)(b-4); b; })

#define FLOAT_MOVE			// Use this to do floating point accuracy for player movement

// ----- Main game variables and operational parameters -----

extern char game_state;
extern char quick_start;
extern char edit_mode;
extern char map_editor;
extern char use_mouse;
extern char use_big_map;
extern int num_initial_units;
extern int num_initial_mainbases;
extern int num_initial_minibases;
extern char debug;
extern char showpath;
extern long heapused;
extern char motion_blur;

// ----- Help Text -----

extern char *helptext;

// ----- Graphics system variables -----

// Rendering method (PAGE_FLIP or DOUBLE_BUFFER)
extern int gfx_method;
// Allegro graphics mode
extern int gfx_mode;
// Physical screen dimensions
extern int PhysicalWidth;
extern int PhysicalHeight;
// Logical screen dimensions
extern int LogicalWidth;
extern int LogicalHeight;
// The world coordinates for the first tile drawn of the background
extern signed int bgx, bgy, bgxoff, bgyoff;
// Variables for keeping track of the screen sliding
extern int bglagx, bglagy, bglagxoff, bglagyoff;
extern char slide_enabled;
extern signed int bgxslide, bgyslide;
// White palette, used for fading involving complete saturation
extern PALETTE whitepal;
// Bitmap definitions for the visible and hidden drawing pages
extern BITMAP *VisiblePage, *HiddenPage;
// Allegro graphics driver name and description, displayed at end of program
extern char gfx_name[80];
extern char gfx_desc[256];
// Default text color (a little bluish white)
//extern int TEXT_COLOR;
// RGB Mapping Table (speeds up calls to makecol())
extern RGB_MAP rgb_table;
// Transparency Mapping Table
extern COLOR_MAP trans_table;

// ----- Player related variables -----

// Main player structure definition
extern player_t player;
// The number of active players (currently unused)
extern int active_players;
// Screen pixel coordinates of the player - constantly reassigned
extern int centerx, centery;
// ID of the unit that the player is currently carrying
extern int unit_carried;
// Afterburner flag
extern char afterburner;
// Invincibility flag
extern int invincible;
// Player starting coordinates (reassigned by main base creation)
extern int plsx, plsy, plsxoff, plsyoff;
// Death stuff
extern long death_start_time;
extern long last_explosion_time;
extern int explosion_rate;
// Shooting stuff (some of this is currently unused)
extern int gun_speed[NUM_GUNS];
extern int gun_status[NUM_GUNS];
extern long gun_rate[NUM_GUNS];
extern long last_gun_time[NUM_GUNS];
// Order state (for ordering units)  -1 if no order, else unit ID # on order
extern signed int order_state;
// Order start time
extern unsigned long order_time;

// Used for the player update interrupt (NOT CURRENTLY USED)
extern fixed player_update_direction, player_update_speed;

// ----- Enemy related variables -----

// # of active enemies
extern char active_enemies;
// Main enemy structure definition
extern player_t enemy[MAX_PLAYERS];

// ----- Projectile and explosion related variables -----

// Projectile declaration
extern projectile_struct proj[MAX_PROJECTILES];
// Explosion declaration
extern explosion_struct explosion[MAX_EXPLOSIONS];
// The number of active projectiles
extern int active_projectiles;
// # of non-active projectiles.. used to find # of truly active ones
extern int non;
// The number of active explosions
extern int active_explosions;
// Coordinate offsets for the explosions (to keep them centered)
// This is necessary because the explosion bitmaps are stored with
// optimal, rather than consistent, bounding boxes.
extern signed char small_exp_xoff[7];
extern signed char small_exp_yoff[7];
extern signed char big_exp_xoff[9];
extern signed char big_exp_yoff[9];

// ----- Map related variables -----

// Map dimensions
extern int MAP_HEIGHT;
extern int MAP_WIDTH;
// Currently used tile graphics filename
extern char tilefile[80];
// Tile map definition (for the visible tile #'s)
// extern char *map;
extern short *map;
// Frill map definition (a layer placed above the normal visible tiles, for bases and such)
extern short *frill;
// Map used for terrain types of each tile
extern signed char *terrain;
// Visibility map
extern char *vis;
// Map used to determine if there are units at any given location
extern thing_t **unit_at;
// Terrain bitmap, used for optimizing the mini-map
extern BITMAP *terrain_bmp;
// Flag for redrawing the terrain bitmap for the mini-map
extern char redraw_terrain_bmp;
// Map object bitmap, used for making radar work with the mini-map
extern BITMAP *mapobj_bmp;
// Visible mini-map bitmap - basically terrain_bmp + mapobj_bmp, updated by radar
extern BITMAP *vismap_bmp;
// Used for drawing walls (the start coordinates)
extern int penx, peny;
// Current map header
extern map_header_t current_map;
// Current map filename
extern char current_map_filename[80];

// ----- Radar related variables -----

// Global radar linked list pointer
extern radar_t *radar;  // top radar in the linked list

// ----- Base and mini-base related variables -----

// Linked list pointers for the master [base and mini-base] lists
extern struct BASE_NODE *base;
extern struct MBASE_NODE *mbase;

// ----- Unit related variables -----

// Global unit linked list pointer
extern unit_t *units;  // top unit in the linked list

// Fresh ID value - this is like an autonumber in Access - just used and incremented
extern int fresh_id;

// Is there a unit ready to be picked up
extern char unit_ready;

// The number of active units
extern int active_units;
// The currently desired terrain- this is set and changed depending on
// what sort of terrain a unit can use.  The initial setting of TERRAIN_LAND
// should not matter (does any code rely on it?)
extern char desired_terrain;
// Unit names
extern char *unit_name[NUM_UNITS];
// The desired terrain types for various units
extern int unit_terrain[NUM_UNITS];
// The time it takes to order various units (in milliseconds)
extern int unit_order_time[NUM_UNITS];
// The cost for each unit ($$$)
extern int unit_cost[NUM_UNITS];
// Range that the unit can see
extern int unit_vis_range[NUM_UNITS];

// ----- GUI related variables -----

// Window list pointer
extern wnd_t *wnd_list;

// ----- File access variables -----

// The main game datafile
extern DATAFILE *datafile;
// The ships datafile
extern DATAFILE *shipgfx;
// The chopper datafile
extern DATAFILE *choppergfx;
// The half-size graphics datafile
extern DATAFILE *halfdatafile;
// The terrain datafile
extern DATAFILE *terraingfx;
// The terrain filename
extern char terrainfile[80];
// The frill gfx datafile
extern DATAFILE *frillgfx;
// The number of tiles in the current tile gfx datafile
extern int NUM_TILES;
// The number of tiles in the current frill gfx datafile
extern int NUM_FRILL_TILES;
// The number of ship types
extern int num_ship_types;

// ----- Talking (messaging) variables -----
extern char talking;			// Is the player currently talking (typing)?
extern char cursor;				// X position of the cursor, in characters
extern char talk_buf[400];		// Storage area for what the player is typing

// ----- Sound, music, and input related variables -----

extern int alt_input;
extern int sound_on;
extern int music_on;
extern int joystick_on;
extern int digi_vol;
extern int midi_vol;
extern MIDI *game_music;

// ----- Statistic tracking variables -----

// Frames-per-second counter variables
extern int fps;
extern int frame_count;
extern long total_frames;
extern int frames;

// ----- Virtual timer variables -----

extern volatile unsigned long ms_time;  // This can be relied upon and will not be reset
extern volatile unsigned long ms_unit;  // This gets reset periodically for unit timing
extern volatile unsigned long ms_proj;  // This gets reset periodically for projectile timing
extern volatile unsigned long ms_fly;   // This gets reset periodically for movement timing
extern volatile unsigned long ms_accel; // This gets reset periodically for acceleration timing
extern volatile unsigned long ms_turn;  // This gets reset periodically for turn timing
extern volatile unsigned long ms_regen; // This gets reset periodically for regeneration
extern volatile unsigned long ms_send;  // This gets reset periodically for sending
// Used to simulate the old timing system based on clock()
extern volatile unsigned long simclock;

// ----- Communications variables -----

// Communications mode - client, server, or no communications at all
extern char comm_mode;
// Connection type - serial (incomplete), Winsock UDP/IP, or none
extern char conn_type;
// Initialization state
extern char init_state;
// Time when the last packet was received (set from ms_time)
extern unsigned long last_packet_time;
// State stream packet send frequencies for player and enemy
extern int send_frequency;
extern int enemy_send_freq;
// For keeping track of how much data has been received.. (for terrain & map downloads)
extern int datarecvd;
extern int packetsrecvd;
// Map packet size
extern int MAP_PACKET_SIZE;

// ----- Terrain stuff -----

extern signed int chunk[6][7];
extern signed int chunk_terrain[6][7];

// Map editor variables

// Tile source window coordinates
////extern int tilesrcx, tilesrcy;
// Tile-under-mouse world coordinates
extern int mtilex, mtiley;
// Mouse locking variable
extern char mlock;
// Flag to hide all windows
extern char hide_all_windows;
// Flag to display terrain type on top of each tile
extern char display_terrain_type;
// Flag to display tile grid lines
extern char display_grid;
