// help.c: Help viewer for Bapple2.  Invoked by hitting F1.  Displays a text file,
// doing primitive word wrapping if necessary, allowing user to smoothly (hopefully)
// scroll up and down.

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <allegro.h>

#include "bapple2.h"
#include "globals.h"

void display_help(void);
void load_help(void);

char helprow[500][80];
int numrows = 0;

int rowoff = 0;
signed char rowyoff = 0;
signed char ydelta = 0;

void display_help(void)
{
	int row, y = 0;
	signed char fh;

	if (!numrows)
		load_help();

	clear(HiddenPage);

	fh = text_height(font);

	row = rowoff;
	if (rowyoff < 0)
		rowyoff = 0;
	for (y = 32 - rowyoff; y < PhysicalHeight + 32 - fh; y += fh + 2)
	{
		if (row >= numrows)
			break;
		textout(HiddenPage, font, helprow[row++], 32, y, TEXT_COLOR);
	}

	if (key[KEY_DOWN])
	{
		ydelta++;
	}
	else
		if (key[KEY_UP])
		{
			if (rowoff > 0)
				ydelta--;
		}
	else
	{
		if (ydelta < 0)
		{
			ydelta += 2;
			if (ydelta > 0)
				ydelta = 0;
		}
		else
			if (ydelta > 0)
			{
				ydelta -= 2;
				if (ydelta < 0)
					ydelta = 0;
			}
	}

	if (ydelta > 20)
		ydelta = 20;
	if (ydelta < -20)
		ydelta = -20;

	rowyoff += ydelta;

	if (rowyoff < 0 - (fh + 2))
	{
		rowyoff = 0;
		rowoff--;
		if (rowoff < 0)
			rowoff = 0;
	}

	if (rowyoff > fh + 2)
	{
		rowyoff = 0;
		rowoff++;
	}

}

void load_help(void)
{
	FILE *fp;
	struct stat s;
	int cr;		// current row
	int bp;		// breaking point
	int i, j, k;
	char done;
	char buf[256];

	fp = fopen("bapple2.txt", "r");
	if (!fp)
		return;
//	fstat(fileno(fp), &s);

//	helptext = (char *) malloc(s.st_size);

	while (!feof(fp))
	{
		cr = numrows;
		fgets(helprow[numrows++], 80, fp);
		// Fancy word/line wrapping
		while (text_length(font, helprow[cr]) > SCREEN_W)
		{
			j = strlen(helprow[cr]) - 1;
			done = bp = 0;
			while (!done)
			{
				// Seek backwards looking for a space
				for (i = j - 1; ((i >= 0) && (helprow[cr][i] != ' ')); i--);
				j = i;
				// We couldn't find a space.. break the line right in the middle
				if (i == 0)
				{
					bp = strlen(helprow[cr]) / 2;
					strcpy(buf, helprow[cr]);
					buf[bp] = '\0';
					done = 1;
				}
				else	// We found a space, let's see if the resulting text is short 'nuf
				{
					strcpy(buf, helprow[cr]);
					buf[j] = '\0';
					if (text_length(font, buf) < SCREEN_W)
					{
						bp = j;
						done = 1;
						break;
					}
				}
			}
			strcpy(helprow[cr], buf);
			strcpy(helprow[numrows++], &helprow[cr][(bp + 1)]);
			cr++;
		}
	}

	fclose(fp);
}
