// map.h: Header file for map related functions used by Bapple2.

#ifndef _MAP_

#define _MAP_

// Map file header

typedef struct {
	int magic;			// Magic number (for file identification purposes)
    int w;				// Map width, in tiles
	int h;				// Map height, in tiles
    char name[80];		// Map name
	char author[80];	// Author
	char tilefile[40];	// Name of tile file (for background tiles) that this map uses
} map_header_t;

// Window struct

typedef struct wnd_t {
	int x, y;				// coordinates of the actual window frame itself
	int intx, inty;			// coordinates of the interior of the window
	int w, h;				// width and height of the actual window frame itself
	int intw, inth;			// width and height of the interior of the window
	char style;				// window style
	char *title;			// window title text
	void (*draw)(struct wnd_t *w);	// window [interior] drawing function pointer
	int (*input)(struct wnd_t *w);	// input handling function pointer

	void *data;				// A pointer to window-specific data (not required)

	struct wnd_t *next;			// pointer to the next window in the window list (or NULL)
} wnd_t;

// Constants for the map editor GUI

// Dimensional definitions for windows

#define BORDER_WIDTH	6	// Border width for a window (left, right, bottom)
#define TITLEBAR_HEIGHT	12	// Title bar height for a window (top)  (Windows uses 22)
#define TOP_HEIGHT		TITLEBAR_HEIGHT + 7		// Total top height for a window...
#define SCROLLBAR_WIDTH	18	// Scrollbar width (applies to both horizontal and vertical)

// Window styles

#define WND_PLAIN		0	// A plain window with no scrollbars
#define WND_HSCROLLBAR	1	// A window with only a horizontal scrollbar
#define WND_VSCROLLBAR	2	// A window with only a vertical scrollbar
#define WND_SCROLLBARS	4	// A window with both horizontal and vertical scrollbars
#define WND_NOTITLE		8	// A window with no title bar

// Button styles

#define BUTTON_PLAIN	0	// A plain button (blank)
#define BUTTON_UP		1	// A button with an arrow pointing up (scrollbar button)
#define BUTTON_DOWN		2	// A button with an arrow pointing down (scrollbar button)
#define BUTTON_LEFT		3	// A button with an arrow pointing left (scrollbar button)
#define BUTTON_RIGHT	4	// A button with an arrow pointing right (scrollbar button)
#define BUTTON_X		5	// A button with an X on it (close window button)

#endif	// #ifndef _MAP_
