// path.c: Pathfinding code (A* algorithm) for Bapple2.

extern unsigned int ms_time;
extern int FreeTile(int x, int y);
extern int TileNum(int x, int y);
extern void add_msg(char type, char *text, ...);

#include <allegro.h>

extern char showpath;
extern int MAP_WIDTH;
extern short *map;
struct InetAddress
{
  short family;
  short port;
  int address;
  char zero [8];
};
typedef struct {
    unsigned long last_shot_time;
    char shot_rate;
    fixed speed;
    fixed maxspeed;
    signed char type;
    int x;
    int y;
    signed int xoff;
    signed int yoff;
    int health;
	int fuel;
    int direction;				// Visible direction
    int targetdir;				// Direction of motion
	char hidden;				// Hide the player from view?  (like when they're dead)
	struct BASE_NODE *base;		// Mini-bases that this player owns
	struct MBASE_NODE *mbase;	// Main-bases that this player owns

	// Comm stuff
	char name[16];				// Name
    unsigned char id;			// ID # (assigned by the server)
	char comm_state;			// Communications state (connected, initializing, etc)
	char addr[80];				// Network address
	unsigned long latency;		// Last time (ms_time) that we heard anything from this player
	unsigned char keepalives;	// Number of keepalives sent to the client so far
	short int kills;			// Number of kills
	short int deaths;			// Number of deaths
	unsigned char lastattacker;	// The ID # of the last person who hurt this player
} player_t;
extern player_t player;

// Macro for accessing a short-int-based linear address space with two subscripts
#define tmpos(a, y, x)	(a)[(((MAP_WIDTH * (y)) + (x)) )]

#ifndef WIN32
#include <mem.h>
#endif

#include "msg.h"
#include "path.h"          // Used for the directional path finding

extern long heapused;

#define calloc(a,b) calloc(a,b), heapused += b
#define malloc(a) malloc(a), heapused += a

/**************************************************************************/
/***************************** A* Algorithm *******************************/
/**************************************************************************/

/* The A* Algorithm is a popular heuristic-based algorithm used for
 * quickly finding a path from one point to another on a 2D grid.
 * It's pretty cool.
 */

/* FindPath(): ------------------------------------------------------------
 *   Given two points, this returns a path to the target as a linked list.
 */
struct NODE *FindPath(long sx,long sy,long dx,long dy)
{
    struct NODE *Node, *BestNode;
    int TileNumDest;
    long starttime;
	int num = 0;
	static int iterations = 0;

    TileNumDest=TileNum((int)dx,(int)dy);

    OPEN=(struct NODE *)calloc(1,sizeof( struct NODE ));
    CLOSED=(struct NODE *)calloc(1,sizeof( struct NODE ));

    Node=(struct NODE *)calloc(1,sizeof( struct NODE ));
    Node->g=0;
    Node->h=(sx-dx)*(sx-dx)+(sy-dy)*(sy-dy);  // should really use sqrt().
    Node->f=Node->g+Node->h;
    Node->NodeNum=TileNum((int)sx,(int)sy);
    Node->x=sx;
    Node->y=sy;

    OPEN->NextNode=Node;        /* make Open List point to first node */

    starttime = ms_time;

//    for (;;)
    while (ms_time - starttime < 200)    // Provide a time-out for hopeless units
    {
        BestNode=(struct NODE *)ReturnBestNode();
        if (BestNode == NULL)
            break;
        if (BestNode->NodeNum == TileNumDest)    /* if we've found the end, break and finish */
           break;
        GenerateSuccessors(BestNode,dx,dy);
		if (showpath)	// special path animation
		{
			num++;
			if (num > iterations)
			{
				iterations++;
				tmpos(map, BestNode->y, BestNode->x) = 2;
				player.x = BestNode->x;
				player.y = BestNode->y;
				return NULL;
			}
		}
    }

	iterations = 0;

	if (showpath)
		add_msg(MSG_COMPUTER, "Pathfind (%d,%d) -> (%d,%d) took %d loops, %dms", (int) sx, (int) sy, (int) dx, (int) dy, num, (int) (ms_time - starttime));

	if (BestNode == NULL)
	{
		if (showpath)
			add_msg(MSG_COMPUTER, "BestNode == NULL!");
		return NULL;
	}

    if (ms_time - starttime >= 200)  // If it's been too long, give up!
	{
		if (showpath)
			add_msg(MSG_COMPUTER, "Gave up on pathfinding after %d loops, %dms!", num, (int) (ms_time - starttime));
        return NULL;
	}

  return (BestNode);
}

struct NODE *ReturnBestNode(void)
{
    struct NODE *tmp;

    if (OPEN->NextNode == NULL)
    {
/*      set_gfx_mode(GFX_TEXT, 80, 25, 0, 0);
        printf("\aA unit was simply UNABLE to calculate a path to its destination!!\n");
        exit(0); */
        return NULL;
    }

/* Pick Node with lowest f, in this case it's the first node in list
   because we sort the OPEN list wrt lowest f. Call it BESTNODE. */

    tmp=OPEN->NextNode;   // point to first node on OPEN
    OPEN->NextNode=tmp->NextNode;    // Make OPEN point to nextnode or NULL.

/* Next take BESTNODE (or temp in this case) and put it on CLOSED */

    tmp->NextNode=CLOSED->NextNode;
    CLOSED->NextNode=tmp;

    return(tmp);
}

void GenerateSuccessors(struct NODE *BestNode,long dx,long dy)
{
  long x,y;

#define TILESIZE 1

            /* Upper-Left  */
    if (FreeTile(x=(long)BestNode->x-TILESIZE,y=(long)BestNode->y-TILESIZE))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Upper       */
    if (FreeTile(x=(long)BestNode->x,y=(long)BestNode->y-TILESIZE))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Upper-Right */
    if (FreeTile(x=(long)BestNode->x+TILESIZE,y=(long)BestNode->y-TILESIZE))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Right       */
    if (FreeTile(x=(long)BestNode->x+TILESIZE,y=(long)BestNode->y))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Lower-Right */
    if (FreeTile(x=(long)BestNode->x+TILESIZE,y=(long)BestNode->y+TILESIZE))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Lower       */
    if (FreeTile(x=(long)BestNode->x,y=(long)BestNode->y+TILESIZE))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Lower-Left  */
    if (FreeTile(x=(long)BestNode->x-TILESIZE,y=(long)BestNode->y+TILESIZE))
      GenerateSucc(BestNode,x,y,dx,dy);
            /* Left        */
    if (FreeTile(x=(long)BestNode->x-TILESIZE,y=(long)BestNode->y))
      GenerateSucc(BestNode,x,y,dx,dy);
}

void GenerateSucc(struct NODE *BestNode,long x, long y, long dx, long dy)
{
    int g,TileNumS,c=0;
    struct NODE *Old,*Successor;

    g=BestNode->g+1;        /* g(Successor)=g(BestNode)+cost of getting from BestNode to Successor */
    TileNumS=TileNum((int)x,(int)y);  /* identification purposes */

    if ((Old=CheckOPEN(TileNumS)) != NULL) /* if equal to NULL then not in OPEN list, else it returns the Node in Old */
    {
    for(c=0;c<8;c++)
      if(BestNode->Child[c] == NULL) /* Add Old to the list of BestNode's Children (or Successors). */
       break;
    BestNode->Child[c]=Old;

    if ((Old) && (g < Old->g))  /* if our new g value is < Old's then reset Old's parent to point to BestNode */
    {
        Old->Parent=BestNode;
        Old->g=g;
        Old->f=g+Old->h;
    }
    }
    else if ((Old=CheckCLOSED(TileNumS)) != NULL) /* if equal to NULL then not in OPEN list, else it returns the Node in Old */
    {
      for(c=0;c<8;c++)
    if (BestNode->Child[c] == NULL) /* Add Old to the list of BestNode's Children (or Successors). */
      break;
    BestNode->Child[c]=Old;

    if (g < Old->g)  /* if our new g value is < Old's then reset Old's parent to point to BestNode */
    {
        Old->Parent=BestNode;
        Old->g=g;
        Old->f=g+Old->h;
        PropagateDown(Old);       /* Since we changed the g value of Old, we need
                     to propagate this new value downwards, i.e.
                     do a Depth-First traversal of the tree! */
    }
    }
    else
    {
    Successor=(struct NODE *)calloc(1,sizeof( struct NODE ));
    Successor->Parent=BestNode;
    Successor->g=g;
    Successor->h=(x-dx)*(x-dx)+(y-dy)*(y-dy);  // should do sqrt(), but since we don't really
    Successor->f=g+Successor->h;     // care about the distance but just which branch looks
    Successor->x=x;                  // better this should suffice. Anyayz it's faster.
    Successor->y=y;
    Successor->NodeNum=TileNumS;
    Insert(Successor);     /* Insert Successor on OPEN list wrt f */
    for(c=0;c<8;c++)
    if (BestNode->Child[c] == NULL) /* Add Old to the list of BestNode's Children (or Successors). */
      break;
    BestNode->Child[c]=Successor;
    }
}

struct NODE *CheckOPEN(int tilenum)
{
    struct NODE *tmp;

    tmp=OPEN->NextNode;
    while (tmp != NULL)
    {
    if (tmp->NodeNum == tilenum)
      return (tmp);
    else
      tmp=tmp->NextNode;
    }
  return (NULL);
}

struct NODE *CheckCLOSED(int tilenum)
{
    struct NODE *tmp;

    tmp=CLOSED->NextNode;

    while (tmp != NULL)
    {
    if (tmp->NodeNum == tilenum)
      return (tmp);
    else
      tmp=tmp->NextNode;
    }
  return (NULL);
}

void Insert(struct NODE *Successor)
{
    struct NODE *tmp1,*tmp2;
    long f;

    if (OPEN->NextNode == NULL)
      {
    OPEN->NextNode=Successor;
    return;
      }

       /* insert into OPEN successor wrt f */

    f=Successor->f;
    tmp1=OPEN;
    tmp2=OPEN->NextNode;

    while ((tmp2 != NULL) && (tmp2->f < f))
    {
      tmp1=tmp2;
      tmp2=tmp2->NextNode;
    }
    Successor->NextNode=tmp2;
    tmp1->NextNode=Successor;
}

void PropagateDown(struct NODE *Old)
{
    int c,g;
    struct NODE *Child, *Father;

    g=Old->g;            // alias.
    for(c=0;c<8;c++)
    {
    if ((Child=Old->Child[c])==NULL)   // create alias for faster access.
      break;
    if (g+1 < Child->g)
    {
      Child->g=g+1;
      Child->f=Child->g+Child->h;
      Child->Parent=Old;           // reset parent to new path.
      Push(Child);                 // Now the Child's branch need to be
    }     // checked out. Remember the new cost must be propagated down.
    }

    while (Stack->NextStackPtr != NULL)
    {
    Father=Pop();
    for(c=0;c<8;c++)
    {
     if ((Child=Father->Child[c])==NULL)       /* we may stop the propagation 2 ways: either */
      break;
      if (Father->g+1 < Child->g) /* there are no children, or that the g value of */
      {                           /* the child is equal or better than the cost we're propagating */
        Child->g=Father->g+1;
        Child->f=Child->g+Child->h;
        Child->Parent=Father;
        Push(Child);
      }
    }
    }
}

/**************************************************************************/
/*                            STACK FUNCTIONS (only for the A* algorithm) */
/**************************************************************************/

void Push(struct NODE *Node)
{
    struct STACK *tmp;

    tmp=( struct STACK *)calloc(1,sizeof(struct STACK));
    tmp->NodePtr=Node;
    tmp->NextStackPtr=Stack->NextStackPtr;
    Stack->NextStackPtr=tmp;
}

struct NODE *Pop()
{
    struct NODE *tmp;
    struct STACK *tmpSTK;

    tmpSTK=Stack->NextStackPtr;
    tmp=tmpSTK->NodePtr;

    Stack->NextStackPtr=tmpSTK->NextStackPtr;
    free(tmpSTK);
    return (tmp);
}
